__author__ = 'Administrator'

import os
import sys
from Bio import SearchIO

PATH = sys.path[0] + '/../'


def _pfam_data():
    pfamA = {}
    data = open(PATH + 'data/Pfam-A.dat')
    for line in data:
        if line.startswith('#=GF ID'):
            name = line[10:].strip()
            if name not in pfamA:
                pfamA[name] = {'NE': {}}
        elif line.startswith('#=GF AC'):
            pfamA[name]['AC'] = line[10:].strip()
        elif line.startswith('#=GF DE'):
            pfamA[name]['DE'] = line[10:].strip()
        elif line.startswith('#=GF GA'):
            values = line[10:].strip().split('; ')
            pfamA[name]['seqGA'] = values[0]
            pfamA[name]['domGA'] = values[1]
        elif line.startswith('#=GF TP'):
            pfamA[name]['TP'] = line[10:].strip()
        elif line.startswith('#=GF ML'):
            pfamA[name]['ML'] = line[10:].strip()
        elif line.startswith('#=GF CL'):
            pfamA[name]['CL'] = line[10:].strip()
        elif line.startswith('#=GF NE'):
            nest = line[10:].strip()
            if nest not in pfamA:
                pfamA[nest] = {'NE': {}}
            pfamA[name]['NE'][nest] = 1
            pfamA[nest]['NE'][name] = 1
    data.close()
    return pfamA


class Scan():
    def __init__(self, db, fasta, outfile=None, overlap=False, E=None, domE=None, T=None, domT=None, cpu=None):
        self.db = db
        self.fasta = fasta
        self.outfile = outfile
        self.overlap = overlap
        self.E = E
        self.domE = domE
        self.T = T
        self.domT = domT
        self.cpu = cpu
        self.records = None

    def search(self):
        params = ['hmmscan', '--notextw']
        cutoff = []
        if self.cpu:
            params += ['--cpu', self.cpu]
        if self.E:
            cutoff += ['-E', self.E]
        if self.domE:
            cutoff += ['--domE', self.domE]
        if self.T:
            cutoff += ['-T', self.T]
        if self.domT:
            cutoff += ['--domT', self.domT]
        if len(cutoff) == 0:
            cutoff += ['--cut_ga']
        params += cutoff
        files = os.listdir(self.db)
        hmmlibs = {'Pfam-A.hmm', 'Pfam-A.hmm.h3f', 'Pfam-A.hmm.h3i', 'Pfam-A.hmm.h3m', 'Pfam-A.hmm.h3p'}
        if not hmmlibs.issubset(files):
            print 'FATAL: cannot find %s and/or %s binaries in %s!' % ('Pfam-A.hmm', 'Pfam-A.hmm', self.db)
            sys.exit(2)
        elif not os.path.isfile(self.fasta):
            print 'FATAL: the fasta file %s does not exist!' % self.fasta
            sys.exit(2)
        else:
            params += [os.path.join(self.db, 'Pfam-A.hmm'), self.fasta]
        hmmscan = os.popen(' '.join(params))
        if hmmscan == None:
            print 'FATAL: hmmscan execution failed. Please check whether HMMER3 has been installed correctly!'
        records = SearchIO.parse(hmmscan, 'hmmer3-text')
        self.records = [record for record in records]
        self._remove_overlap()

    def _remove_overlap(self):
        pfamA = _pfam_data()
        results = []
        for record in self.records:
            units = []
            hsps = record.hsps
            hsps.sort(cmp=None, key=lambda x: x.evalue_cond, reverse=False)
            for hsp in hsps:
                flag = False
                for unit in units:
                    if 'CL' in pfamA[hsp.hit_id] and 'CL' in pfamA[unit.hit_id] and \
                                    pfamA[hsp.hit_id]['CL'] == pfamA[unit.hit_id]['CL']:
                        hsp_range = set(range(hsp.query_start, hsp.query_end))
                        unit_range = set(range(unit.query_start, unit.query_end))
                        if not (hsp_range.isdisjoint(unit_range) or unit.hit_id in pfamA[hsp.hit_id]['NE']):
                            flag = True
                            break
                if not flag:
                    hsp.acc = pfamA[hsp.hit_id]['AC']
                    hsp.length = int(pfamA[hsp.hit_id]['ML'])
                    hsp.describe = pfamA[hsp.hit_id]['DE']
                    units.append(hsp)
            units.sort(cmp=None, key=lambda x: x.query_start, reverse=False)
            results.append(units)
        self.records = results

# if __name__ == '__main__':
#     scan = Scan(PATH + 'data', PATH + 'example/P16157.fasta')
#     scan.search()